"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.printOutboxPollingListenerEnvVariables = exports.printInboxPollingListenerEnvVariables = exports.getOutboxPollingListenerSettings = exports.getInboxPollingListenerSettings = exports.applyDefaultPollingListenerConfigValues = void 0;
const env_settings_1 = require("../common/env-settings");
const listener_config_1 = require("../common/listener-config");
const defaultSettings = {
    nextMessagesFunctionSchema: 'public',
    nextMessagesBatchSize: 5,
    nextMessagesLockInMs: 5000,
    nextMessagesPollingIntervalInMs: 500,
};
const applyDefaultPollingListenerConfigValues = (config) => {
    const listenerConfig = (0, listener_config_1.applyDefaultListenerConfigValues)(config);
    const filledConfig = Object.assign(Object.assign(Object.assign({}, listenerConfig), config), { settings: Object.assign(Object.assign(Object.assign({}, listenerConfig.settings), defaultSettings), config.settings) });
    return filledConfig;
};
exports.applyDefaultPollingListenerConfigValues = applyDefaultPollingListenerConfigValues;
const basicSettingsMap = [
    {
        constantName: 'NEXT_MESSAGES_FUNCTION_SCHEMA',
        default: defaultSettings.nextMessagesFunctionSchema,
        func: env_settings_1.getEnvVariableString,
    },
    {
        constantName: 'NEXT_MESSAGES_BATCH_SIZE',
        default: defaultSettings.nextMessagesBatchSize,
        func: env_settings_1.getEnvVariableNumber,
    },
    {
        constantName: 'NEXT_MESSAGES_LOCK_IN_MS',
        default: defaultSettings.nextMessagesLockInMs,
        func: env_settings_1.getEnvVariableNumber,
    },
    {
        constantName: 'NEXT_MESSAGES_POLLING_INTERVAL_IN_MS',
        default: defaultSettings.nextMessagesPollingIntervalInMs,
        func: env_settings_1.getEnvVariableNumber,
    },
];
const inboxSettingsMap = [
    {
        constantName: 'NEXT_MESSAGES_FUNCTION_NAME',
        default: 'next_inbox_messages',
        func: env_settings_1.getEnvVariableString,
        skipFallback: true,
    },
];
const outboxSettingsMap = [
    {
        constantName: 'NEXT_MESSAGES_FUNCTION_NAME',
        default: 'next_outbox_messages',
        func: env_settings_1.getEnvVariableString,
        skipFallback: true,
    },
];
/**
 * Loads the environment variables into the polling listener settings object. It
 * supports reading an inbox specific setting or a general one.
 * Please use the `printInboxPollingListenerEnvVariables` functions to get a
 * list of all the inbox relevant settings for the polling listener.
 * @example
 * TRX_DB_SCHEMA=trx_schema
 * TRX_INBOX_DB_TABLE=inbox_table
 * TRX_INBOX_NEXT_MESSAGES_FUNCTION_SCHEMA=next_inbox_messages
 * @param env The process.env variable or a custom object.
 * @returns The polling listener settings object with filled with the ENV variables
 */
const getInboxPollingListenerSettings = (env = process.env) => {
    const inboxSettings = (0, listener_config_1.getInboxListenerSettings)(env);
    const pollSettings = (0, env_settings_1.getConfigSettings)([...basicSettingsMap, ...inboxSettingsMap], listener_config_1.inboxEnvPrefix, listener_config_1.fallbackEnvPrefix, env);
    return Object.assign(Object.assign({}, inboxSettings), pollSettings);
};
exports.getInboxPollingListenerSettings = getInboxPollingListenerSettings;
/**
 * Loads the environment variables into the polling listener settings object. It
 * supports reading an outbox specific setting or a general one.
 * Please use the `printOutboxPollingListenerEnvVariables` functions to get a
 * list of all the outbox relevant settings for the polling listener.
 * @example
 * TRX_DB_SCHEMA=trx_schema
 * TRX_OUTBOX_DB_TABLE=outbox_table
 * TRX_OUTBOX_NEXT_MESSAGES_FUNCTION_SCHEMA=next_outbox_messages
 * @param env The process.env variable or a custom object.
 * @returns The polling listener settings object with filled with the ENV variables
 */
const getOutboxPollingListenerSettings = (env = process.env) => {
    const outboxSettings = (0, listener_config_1.getOutboxListenerSettings)(env);
    const pollSettings = (0, env_settings_1.getConfigSettings)([...basicSettingsMap, ...outboxSettingsMap], listener_config_1.outboxEnvPrefix, listener_config_1.fallbackEnvPrefix, env);
    return Object.assign(Object.assign({}, outboxSettings), pollSettings);
};
exports.getOutboxPollingListenerSettings = getOutboxPollingListenerSettings;
/**
 * Shows the available env variables and their default values for the inbox
 * listener with the polling approach.
 * @param map A mapping of all the env variables to config settings.
 * @param envPrefix The prefix for the env variables to check first (e.g. "TRX_INBOX_" or "TRX_").
 * @param envPrefixFallback The fallback prefix if the other is not found. Useful for defining settings that should be used for both outbox and inbox.
 * @returns
 */
const printInboxPollingListenerEnvVariables = (defaultOverrides) => {
    const il = (0, listener_config_1.printInboxListenerEnvVariables)(defaultOverrides);
    const cfg = (0, env_settings_1.printConfigSettings)([...basicSettingsMap, ...inboxSettingsMap], listener_config_1.inboxEnvPrefix, listener_config_1.fallbackEnvPrefix, defaultOverrides);
    return `# Inbox listener variables
${il}
# Inbox polling listener variables
${cfg}`;
};
exports.printInboxPollingListenerEnvVariables = printInboxPollingListenerEnvVariables;
/**
 * Shows the available env variables and their default values for the outbox
 * listener with the polling approach.
 * @param map A mapping of all the env variables to config settings.
 * @param envPrefix The prefix for the env variables to check first (e.g. "TRX_OUTBOX_" or "TRX_").
 * @param envPrefixFallback The fallback prefix if the other is not found. Useful for defining settings that should be used for both outbox and inbox.
 * @returns
 */
const printOutboxPollingListenerEnvVariables = (defaultOverrides) => {
    const ol = (0, listener_config_1.printOutboxListenerEnvVariables)(defaultOverrides);
    const cfg = (0, env_settings_1.printConfigSettings)([...basicSettingsMap, ...outboxSettingsMap], listener_config_1.outboxEnvPrefix, listener_config_1.fallbackEnvPrefix, defaultOverrides);
    return `# Outbox listener variables
${ol}
# Outbox polling listener variables
${cfg}`;
};
exports.printOutboxPollingListenerEnvVariables = printOutboxPollingListenerEnvVariables;
//# sourceMappingURL=config.js.map