#! /usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dbSetupCli = void 0;
const promises_1 = require("node:fs/promises");
const readline_1 = require("readline");
const config_1 = require("../polling/config");
const config_2 = require("../replication/config");
const database_setup_exporter_1 = require("./database-setup-exporter");
const { createPollingScript, createReplicationScript } = database_setup_exporter_1.DatabaseSetupExporter;
/** Async way to ask a question from the CLI */
const input = (prompt, rli) => {
    return new Promise((callbackFn) => {
        rli.question(prompt, (userInput) => {
            callbackFn(userInput === null || userInput === void 0 ? void 0 : userInput.trim());
        });
    });
};
/** Get a value from the command line with allowed values and a default value */
const getValueFromInput = (rli, prompt, allowedAnswers, defaultValue) => __awaiter(void 0, void 0, void 0, function* () {
    let answer;
    do {
        const d = defaultValue ? ` (default: ${defaultValue})` : '';
        answer = yield input(`\x1b[32m${prompt}\x1b[0m${d}\n> `, rli);
        if (defaultValue && !answer) {
            answer = defaultValue;
        }
    } while (allowedAnswers ? allowedAnswers.indexOf(answer) < 0 : !answer);
    return answer;
});
/** Get config values for the logical replication listener */
const logicalReplicationValues = (outboxOrInbox, rli) => __awaiter(void 0, void 0, void 0, function* () {
    const replicationSlot = yield getValueFromInput(rli, `What should the name of the replication slot for the ${outboxOrInbox} be?`, undefined, `transactional_${outboxOrInbox}_slot`);
    const publication = yield getValueFromInput(rli, `What should the name of the replication publication for the ${outboxOrInbox} be?`, undefined, `transactional_${outboxOrInbox}_publication`);
    return { replicationSlot, publication };
});
/** Get config values for the polling listener */
const pollingValues = (outboxOrInbox, schema, rli) => __awaiter(void 0, void 0, void 0, function* () {
    const nextMessagesName = yield getValueFromInput(rli, `What should the name of the ${outboxOrInbox} polling function be?`, undefined, `next_${outboxOrInbox}_messages`);
    const nextMessagesSchema = yield getValueFromInput(rli, `What should the name of the database schema for the "${nextMessagesName}" function be?`, undefined, schema);
    return { nextMessagesName, nextMessagesSchema };
});
/** Execute the CLI */
const dbSetupCli = () => __awaiter(void 0, void 0, void 0, function* () {
    const rli = (0, readline_1.createInterface)({
        input: process.stdin,
        output: process.stdout,
    });
    const getValue = (prompt, allowedAnswers, defaultValue) => getValueFromInput(rli, prompt, allowedAnswers, defaultValue);
    const listener = yield getValue('What type of listener should be created?\n- Logical Replication (r)\n- Polling (p)', ['r', 'p']);
    const database = yield getValue('What is the name of your database?');
    const schema = yield getValue('What should the name of the database schema be?', undefined, 'messaging');
    const listenerRole = yield getValue('What is the name of your database role for the listener?', undefined, 'messaging_listener');
    const handlerRole = yield getValue('What is the name of your database role for the message handler?', undefined, listenerRole);
    const outboxInboxBoth = yield getValue('What transactional types should be created?\n- outbox (o)\n- inbox (i)\n- both (b)', ['o', 'i', 'b']);
    const buildReplicationSql = (outboxOrInbox, table, publication, replicationSlot, skipRoles = false) => __awaiter(void 0, void 0, void 0, function* () {
        const config = {
            database,
            schema,
            table,
            listenerRole,
            handlerRole,
            outboxOrInbox,
            publication,
            replicationSlot,
        };
        return createReplicationScript(config, skipRoles);
    });
    const buildPollingSql = (outboxOrInbox, table, nextMessagesName, nextMessagesSchema, skipRoles = false) => __awaiter(void 0, void 0, void 0, function* () {
        const config = {
            database,
            schema,
            table,
            listenerRole,
            handlerRole,
            outboxOrInbox,
            nextMessagesName,
            nextMessagesSchema,
        };
        return createPollingScript(config, skipRoles);
    });
    const envOverrides = { DB_SCHEMA: schema };
    let envConfig = '';
    let sqlOutput = '';
    if (outboxInboxBoth === 'o' || outboxInboxBoth === 'b') {
        const outboxOrInbox = 'outbox';
        const table = yield getValue('What name should the outbox table have?', undefined, outboxOrInbox);
        if (listener === 'r') {
            const { publication, replicationSlot } = yield logicalReplicationValues(outboxOrInbox, rli);
            sqlOutput += yield buildReplicationSql(outboxOrInbox, table, publication, replicationSlot);
            envConfig += (0, config_2.printOutboxReplicationListenerEnvVariables)(Object.assign(Object.assign({}, envOverrides), { TRX_OUTBOX_DB_TABLE: table, DB_PUBLICATION: publication, DB_REPLICATION_SLOT: replicationSlot }));
        }
        else {
            const { nextMessagesName, nextMessagesSchema } = yield pollingValues(outboxOrInbox, schema, rli);
            sqlOutput += yield buildPollingSql(outboxOrInbox, table, nextMessagesName, nextMessagesSchema);
            envConfig += (0, config_1.printOutboxPollingListenerEnvVariables)(Object.assign(Object.assign({}, envOverrides), { TRX_OUTBOX_DB_TABLE: table, NEXT_MESSAGES_FUNCTION_SCHEMA: nextMessagesSchema, OUTBOX_NEXT_MESSAGES_FUNCTION_NAME: nextMessagesName }));
        }
    }
    if (outboxInboxBoth === 'i' || outboxInboxBoth === 'b') {
        const outboxOrInbox = 'inbox';
        const table = yield getValue('What name should the inbox table have?', undefined, outboxOrInbox);
        if (listener === 'r') {
            const { publication, replicationSlot } = yield logicalReplicationValues(outboxOrInbox, rli);
            sqlOutput += yield buildReplicationSql(outboxOrInbox, table, publication, replicationSlot, outboxInboxBoth === 'b');
            envConfig += (0, config_2.printInboxReplicationListenerEnvVariables)(Object.assign(Object.assign({}, envOverrides), { TRX_INBOX_DB_TABLE: table, DB_PUBLICATION: publication, DB_REPLICATION_SLOT: replicationSlot }));
        }
        else {
            const { nextMessagesName, nextMessagesSchema } = yield pollingValues(outboxOrInbox, schema, rli);
            sqlOutput += yield buildPollingSql(outboxOrInbox, table, nextMessagesName, nextMessagesSchema, outboxInboxBoth === 'b');
            envConfig += (0, config_1.printInboxPollingListenerEnvVariables)(Object.assign(Object.assign({}, envOverrides), { TRX_INBOX_DB_TABLE: table, NEXT_MESSAGES_FUNCTION_SCHEMA: nextMessagesSchema, INBOX_NEXT_MESSAGES_FUNCTION_NAME: nextMessagesName }));
        }
    }
    const filename = yield getValue('What should the filename without extension for the SQL script (*.sql) and the config (*.env) be?', undefined, 'transactional');
    yield (0, promises_1.writeFile)(`${filename}.sql`, sqlOutput);
    // Write the .env File
    envConfig = `# Select the variables that you want to adjust and copy them to your .ENV file/store
# You can leave/skip the config variables that are already fine.
# The defaults will be applied automatically for them.
# If you use both outbox and inbox you can use the shared "TRX_***" variables or
# the outbox and inbox specific "TRX_OUTBOX_***" and "TRX_INBOX_***" ones.

${sortEnv(envConfig)}`;
    yield (0, promises_1.writeFile)(`${filename}.env`, envConfig);
    console.log(`File \x1b[92m${filename}\x1b[0m successfully created.`);
    rli.close();
});
exports.dbSetupCli = dbSetupCli;
const sortEnv = (envConfig) => {
    const lines = envConfig.split('\n');
    const genericLines = lines.filter((l, i, all) => !l.startsWith('TRX_INBOX_') &&
        !l.startsWith('TRX_OUTBOX_') &&
        !l.startsWith('#') &&
        i === all.indexOf(l));
    const outboxLines = lines.filter((l) => l.startsWith('TRX_OUTBOX_'));
    const inboxLines = lines.filter((l) => l.startsWith('TRX_INBOX_'));
    return `# General settings for both outbox and inbox
${genericLines.join('\n')}
# Outbox specific settings - overrides the general settings
${outboxLines.join('\n')}
# Inbox specific settings - overrides the general settings
${inboxLines.join('\n')}`;
};
//# sourceMappingURL=db-setup-cli.js.map